==> foundation-0.0.52.tgz: package/Textures/TextureBuilder.js <==
import { URITextureFormat, URITextureSamplingMode, URITextureTypes, } from "@divinestar/uri/Constants/URITexturesConstants";
import { DivineVoxelEngineRender } from "@divinevoxel/core/Contexts/Render/DivineVoxelEngineRender";
export const TextureBuilder = {
    context: null,
    _textureSize: 16,
    imgWidth: 16,
    imgHeight: 16,
    _canvas: document.createElement("canvas"),
    defineTextureDimensions(textureSize, mipMapSizes) {
        this.imgWidth = textureSize < 256 ? 256 : textureSize;
        this.imgHeight = textureSize < 256 ? 256 : textureSize;
        this._textureSize = textureSize;
    },
    setUpImageCreation() {
        this._canvas.width = this.imgWidth;
        this._canvas.height = this.imgHeight;
        const context = this._canvas.getContext("2d", { willReadFrequently: true });
        if (!context) {
            throw new Error("Context did not load for texture creation.");
        }
        context.imageSmoothingEnabled = true;
        context.imageSmoothingQuality = "high";
        this.context = context;
    },
    async createMaterialTexture(name, images, width = -1, height = -1) {
        if (width == -1)
            width = this.imgWidth;
        if (height == -1)
            height = this.imgHeight;
        this._canvas.width = this.imgWidth;
        this._canvas.height = this.imgHeight;
        return (await this._create(name, images, width, height));
        /*    for (const size of this._mipMapSizes) {
          const texture = await this._createOld(name, images, size, size);
          textures.push(texture);
        }
        return textures; */
    },
    async _create(name, images, width, height) {
        // const scene = RenderManager.scene!;
        const data = (await this._createMipMap(0, images, width, height));
        const texture = DivineVoxelEngineRender.instance.renderer.engine.createTexture({
            type: URITextureTypes.Texture2DArray,
            data,
            width,
            height,
            scene: DivineVoxelEngineRender.instance.renderer.scene,
            layers: images.size + 2,
            format: URITextureFormat.Rgba,
            samplingMode: URITextureSamplingMode.NearestNearestMipLinear,
        });
        // texture.anisotropicFilteringLevel = 16;
        /*     texture._noMipmap = false;
        const iTexture = texture._texture!;
        iTexture.generateMipMaps = true;
        iTexture.useMipMaps = true;
        engine._bindTextureDirectly(engine._gl.TEXTURE_2D_ARRAY, iTexture);
    
        let w = width,
          h = height,
          mipMapLevel = 0;
        while (w >= 1 && h >= 1) {
          if (mipMapLevel < 3) {
            this.context!.imageSmoothingEnabled = false;
          } else {
            this.context!.imageSmoothingEnabled = true;
          }
          const mip = await this._createMipMap(mipMapLevel, images, w, h);
    
          const gl = engine._gl;
          const textureType = engine._getWebGLTextureType(iTexture.type);
          const format = engine._getInternalFormat(iTexture.format);
          const internalFormat = engine._getRGBABufferInternalSizedFormat(
            iTexture.type,
            iTexture.format,
            iTexture._useSRGBBuffer
          );
    
          engine._unpackFlipY(texture.invertY);
    
          console.log(w, h);
          let target = gl.TEXTURE_2D_ARRAY;
    
          gl.texImage3D(
            target,
            mipMapLevel,
            internalFormat,
            w,
            h,
            images.size + 2,
            0,
            format,
            textureType,
            mip
          );
          w /= 2;
          h /= 2;
          mipMapLevel++;
        }
    
        iTexture.width = width;
        iTexture.height = height;
        iTexture.isReady = true;
        //  iTexture.samplingMode = Texture.NEAREST_NEAREST_MIPLINEAR;
        engine._bindTextureDirectly(engine._gl.TEXTURE_2D_ARRAY, null);
        texture._texture = iTexture;
     */
        return texture;
    },
    async _createMipMap(level, images, width, height) {
        //   const scene = RenderManager.scene!;
        const resolvedImages = [];
        //create blank fill to pad image array buffer
        const data = [];
        for (let i = 0; i < width; i++) {
            for (let j = 0; j < height; j++) {
                data.push(0, 0, 0, 1);
            }
        }
        resolvedImages.push(new Uint8ClampedArray(data));
        for (const [path, rawData] of images) {
            const data = await this.loadImage(rawData ? rawData : path, width, height, level);
            resolvedImages.push(data);
        }
        resolvedImages.push(new Uint8ClampedArray(data));
        let totalLength = 0;
        for (const image of resolvedImages) {
            totalLength += image.byteLength;
        }
        return this._combineImageData(totalLength, resolvedImages);
    },
    loadImage(imgSrcData, width = 0, height = 0, lod = 0) {
        if (!width)
            width = this.imgWidth;
        if (!height)
            height = this.imgHeight;
        const ctx = TextureBuilder.context;
        if (!ctx) {
            throw new Error("Context is not set for texture creation.");
        }
        if (typeof imgSrcData == "string") {
            const prom = new Promise((resolve) => {
                const image = new Image();
                image.src = imgSrcData;
                image.onload = async () => {
                    ctx.clearRect(0, 0, this._canvas.width, this._canvas.height);
                    ctx.drawImage(image, 0, 0, image.width, image.height);
                    const imgData = ctx.getImageData(0, 0, image.width, image.height);
                    const bitmap = await createImageBitmap(new ImageData(imgData.data, image.width, image.height), {
                        resizeWidth: width,
                        resizeHeight: height,
                        resizeQuality: lod < 3 ? "pixelated" : "high",
                        premultiplyAlpha: lod < 3 ? "none" : "premultiply",
                    });
                    ctx.clearRect(0, 0, this._canvas.width, this._canvas.height);
                    ctx.drawImage(bitmap, 0, 0, width, height);
                    const bitmapData = ctx.getImageData(0, 0, width, height);
                    //clear the canvas before re-rendering another image
                    resolve(bitmapData.data);
                };
            });
            return prom;
        }
        if (imgSrcData instanceof Uint8ClampedArray) {
            const prom = new Promise(async (resolve) => {
                ctx.fillRect(0, 0, this._canvas.width, this._canvas.height);
                ctx.clearRect(0, 0, this._canvas.width, this._canvas.height);
                const bitmap = await createImageBitmap(new ImageData(imgSrcData, Math.sqrt(imgSrcData.length / 4), Math.sqrt(imgSrcData.length / 4)), {
                    resizeWidth: width,
                    resizeHeight: height,
                    resizeQuality: lod < 3 ? "pixelated" : "high",
                    premultiplyAlpha: lod < 3 ? "none" : "premultiply",
                });
                ctx.drawImage(bitmap, 0, 0, width, height);
                const imgData = ctx.getImageData(0, 0, width, height);
                resolve(imgData.data);
            });
            return prom;
        }
        throw new Error("Context is not set for texture creation.");
    },
    _combineImageData(totalLength, arrays) {
        const combinedImagedata = new Uint8ClampedArray(totalLength);
        const length = arrays[0].length;
        for (let i = 0; i < arrays.length; i++) {
            const array = arrays[i];
            const previousArrayIndex = length * i;
            combinedImagedata.set(array, previousArrayIndex);
        }
        return combinedImagedata;
    },
};

==> foundation-0.0.53.tgz: package/Textures/TextureBuilder.js <==
import { URITextureFormat, URITextureSamplingMode, URITextureTypes, } from "@amodx/uri/Constants/URITexturesConstants";
import { DivineVoxelEngineRender } from "@divinevoxel/core/Contexts/Render/DivineVoxelEngineRender";
export class TextureBuilder {
    static context = null;
    static _textureSize = 16;
    static finalImagWidth = 16;
    static finalImageHeight = 16;
    static _canvas = document.createElement("canvas");
    static defineTextureDimensions(textureSize, mipMapSizes) {
        this.finalImagWidth = textureSize < 256 ? 256 : textureSize;
        this.finalImageHeight = textureSize < 256 ? 256 : textureSize;
        this._textureSize = textureSize;
    }
    static setUpImageCreation() {
        this._canvas.width = this.finalImagWidth;
        this._canvas.height = this.finalImageHeight;
        const context = this._canvas.getContext("2d", { willReadFrequently: true });
        if (!context) {
            throw new Error("Context did not load for texture creation.");
        }
        context.imageSmoothingEnabled = false;
        context.imageSmoothingQuality = "high";
        this.context = context;
    }
    static async createMaterialTexture(name, images, width = -1, height = -1) {
        if (width == -1)
            width = this.finalImagWidth;
        if (height == -1)
            height = this.finalImageHeight;
        this._canvas.width = this.finalImagWidth;
        this._canvas.height = this.finalImageHeight;
        return (await this._create(name, images, width, height));
        /*    for (const size of this._mipMapSizes) {
          const texture = await this._createOld(name, images, size, size);
          textures.push(texture);
        }
        return textures; */
    }
    static async _create(name, images, width, height) {
        // const scene = RenderManager.scene!;
        const data = (await this._createMipMap(0, images, width, height));
        const texture = DivineVoxelEngineRender.instance.renderer.engine.createTexture({
            type: URITextureTypes.Texture2DArray,
            data,
            width,
            height,
            scene: DivineVoxelEngineRender.instance.renderer.scene,
            layers: images.size + 2,
            format: URITextureFormat.Rgba,
            samplingMode: URITextureSamplingMode.TrilinearSamplingMode,
            //@ts-ignore
            images,
        });
        await texture._create();
        // texture.anisotropicFilteringLevel = 16;
        /*     texture._noMipmap = false;
        const iTexture = texture._texture!;
        iTexture.generateMipMaps = true;
        iTexture.useMipMaps = true;
        engine._bindTextureDirectly(engine._gl.TEXTURE_2D_ARRAY, iTexture);
    
        let w = width,
          h = height,
          mipMapLevel = 0;
        while (w >= 1 && h >= 1) {
          if (mipMapLevel < 3) {
            this.context!.imageSmoothingEnabled = false;
          } else {
            this.context!.imageSmoothingEnabled = true;
          }
          const mip = await this._createMipMap(mipMapLevel, images, w, h);
    
          const gl = engine._gl;
          const textureType = engine._getWebGLTextureType(iTexture.type);
          const format = engine._getInternalFormat(iTexture.format);
          const internalFormat = engine._getRGBABufferInternalSizedFormat(
            iTexture.type,
            iTexture.format,
            iTexture._useSRGBBuffer
          );
    
          engine._unpackFlipY(texture.invertY);
    
          console.log(w, h);
          let target = gl.TEXTURE_2D_ARRAY;
    
          gl.texImage3D(
            target,
            mipMapLevel,
            internalFormat,
            w,
            h,
            images.size + 2,
            0,
            format,
            textureType,
            mip
          );
          w /= 2;
          h /= 2;
          mipMapLevel++;
        }
    
        iTexture.width = width;
        iTexture.height = height;
        iTexture.isReady = true;
        //  iTexture.samplingMode = Texture.NEAREST_NEAREST_MIPLINEAR;
        engine._bindTextureDirectly(engine._gl.TEXTURE_2D_ARRAY, null);
        texture._texture = iTexture;
     */
        return texture;
    }
    static async _createMipMap(level, images, width, height) {
        //   const scene = RenderManager.scene!;
        const resolvedImages = [];
        //create blank fill to pad image array buffer
        const data = [];
        for (let i = 0; i < width; i++) {
            for (let j = 0; j < height; j++) {
                data.push(0, 0, 0, 1);
            }
        }
        resolvedImages.push(new Uint8ClampedArray(data));
        for (const [path, rawData] of images) {
            const data = await this.loadImage(rawData ? rawData : path, width, height, level);
            resolvedImages.push(data);
        }
        resolvedImages.push(new Uint8ClampedArray(data));
        let totalLength = 0;
        for (const image of resolvedImages) {
            totalLength += image.byteLength;
        }
        return this._combineImageData(totalLength, resolvedImages);
    }
    static async loadImage(imgSrcData, width = 0, height = 0, lod = 0, flip = true) {
        let testing = false;
        if (!width)
            width = this.finalImagWidth;
        if (!height)
            height = this.finalImageHeight;
        const ctx = TextureBuilder.context;
        if (!ctx) {
            throw new Error("Context is not set for texture creation.");
        }
        const turnRed = (data) => {
            for (let i = 0; i < data.length; i += 4) {
                data[i] = 255; // Red
                data[i + 1] = 0; // Green
                data[i + 2] = 0; // Blue
            }
        };
        if (typeof imgSrcData == "string") {
            const prom = new Promise((resolve) => {
                const image = new Image();
                image.src = imgSrcData;
                image.onload = async () => {
                    ctx.clearRect(0, 0, this._canvas.width, this._canvas.height);
                    ctx.save();
                    if (flip) {
                        ctx.scale(1, -1);
                        ctx.drawImage(image, 0, -image.height, image.width, image.height);
                    }
                    else {
                        ctx.drawImage(image, 0, 0, image.width, image.height);
                    }
                    ctx.restore();
                    const imgData = ctx.getImageData(0, 0, image.width, image.height);
                    const bitmap = await createImageBitmap(new ImageData(imgData.data, image.width, image.height), {
                        resizeWidth: width,
                        resizeHeight: height,
                        resizeQuality: "pixelated",
                        premultiplyAlpha: lod != 0 ? "premultiply" : undefined,
                    });
                    ctx.clearRect(0, 0, this._canvas.width, this._canvas.height);
                    ctx.drawImage(bitmap, 0, 0, width, height);
                    const bitmapData = ctx.getImageData(0, 0, width, height);
                    if (testing && lod > 0) {
                        turnRed(bitmapData.data);
                    }
                    resolve(bitmapData.data);
                };
            });
            return prom;
        }
        if (imgSrcData instanceof Uint8ClampedArray) {
            const prom = new Promise(async (resolve) => {
                ctx.fillRect(0, 0, this._canvas.width, this._canvas.height);
                ctx.clearRect(0, 0, this._canvas.width, this._canvas.height);
                const bitmap = await createImageBitmap(new ImageData(imgSrcData, Math.sqrt(imgSrcData.length / 4), Math.sqrt(imgSrcData.length / 4)), {
                    resizeWidth: width,
                    resizeHeight: height,
                    resizeQuality: "pixelated",
                    premultiplyAlpha: lod != 0 ? "premultiply" : undefined,
                });
                ctx.save();
                if (flip) {
                    ctx.scale(1, -1);
                    ctx.drawImage(bitmap, 0, -height, width, height);
                }
                else {
                    ctx.drawImage(bitmap, 0, 0, width, height);
                }
                ctx.restore();
                const imgData = ctx.getImageData(0, 0, width, height);
                if (testing && lod > 0) {
                    turnRed(imgData.data);
                }
                resolve(imgData.data);
            });
            return prom;
        }
        throw new Error("Context is not set for texture creation.");
    }
    static _combineImageData(totalLength, arrays) {
        const combinedImagedata = new Uint8ClampedArray(totalLength);
        const length = arrays[0].length;
        for (let i = 0; i < arrays.length; i++) {
            const array = arrays[i];
            const previousArrayIndex = length * i;
            combinedImagedata.set(array, previousArrayIndex);
        }
        return combinedImagedata;
    }
}
